/* Animation von Backtracking in einem Labyrinth  */
/* benoetigt Java 1.1.7B oder hoeher              */
/*                                                */
/* (c) Thomas Duebendorfer, Juli 2000             */
/* Last modification: May 16, 2010                */
/* Fuer Lehrzwecke frei verwendbar und anpassbar. */
/* Kommerzielle Verwendung verboten.              */
/*                                                */

import java.awt.*;
import java.awt.event.*;
import java.util.Date;

public class Labyrinth extends Frame implements ActionListener, ItemListener {

// Variablen
private MenuItem newLabyrinth, resetLabyrinth, solve, quit, about;
private Menu AnimationMenu, LabyrinthMenu, DelayMenu, HelpMenu;
private CheckboxMenuItem[] size, delay;
private LabyrinthCanvas labyrinthCanvas;
private MazeGenerator mazeGenerator;
private boolean inAnApplet = true;

Lsg labyrinth, loesung;
int startX, startY, zielX, zielY;
int lastX, lastY;

// Labyrinth Groesse KxL
final static int MINSIZE = 7, MAXSIZE = 25; // Minimum und Maximum
int K=12, L=12; // default

// Animationsverzgerung
final static int MINDELAY = 0, DELTADELAY = 50, MAXDELAY = 800; // Minimum und Maximum
int cur_delay = 200;

// Codierung der Wnde: oben, rechts, unten, links = Bit 1,2,3,4
final int NOWALL = 0x00;
final int UP = 0x01, RIGHT = 0x02, DOWN = 0x04, LEFT = 0x08;
// Codierung der Richtungen
final static int OBEN = 0, RECHTS = 1, UNTEN = 2, LINKS = 3;

/*** Code aus Vortrag zu Backtracking (einzig erweitert um Visualisierung) ***/
final static int LEER = 0, SACKGASSE = -1;  // Feldinformation
final static int[] STEPX = {  0, 1, 0,-1 }; // STEPX,-Y: Schritte 
final static int[] STEPY = { -1, 0, 1, 0 }; // in alle vier Richtungen

boolean FindeLoesung(int index, Lsg loesung, int aktX, int aktY) {
	// index       = aktuelle Schrittzahl
	// loesung     = Referenz auf bisherige Teil-Lsung
	// aktX, aktY  = aktuelle Feldposition im Labyrinth
	int schritt = -1;

	// while(es gibt es noch neue Teil-Lsungsschritte)
	while (schritt != LINKS) {
		// Whle einen neuen Teil-Lsungsschritt schritt;
		schritt ++;	// Weg nach 1. oben, 2. rechts, 3. unten
					// und 4. links zu erweitern versuchen.
		int neuX	= aktX + STEPX[schritt];
		int neuY	= aktY + STEPY[schritt];

		// Tests, ob schritt gltig ist
		boolean ok = true;
		// Test, ob schritt innerhalb Brett bleibt
		if (neuX < 0 || neuX >= K) ok = false;
		if (neuY < 0 || neuY >= L) ok = false;
		// Test, ob schritt durch Wand fhrt (sofern innerhalb)
		if (ok && hatWand(aktX,aktY,neuX,neuY,schritt)) ok = false;
		// Test, ob schritt auf ein bereits besuchtes Feld fhrt
		if (ok && loesung.feld[neuX][neuY] != LEER) ok = false;
 
		// if (schritt ist gltig)
		if (ok) {
			// Erweitere loesung um schritt
			// Markiere neues Feld mit aktueller Schrittzahl
			loesung.feld[neuX][neuY] = index;

			// Visualisierung
			visualisiereFindeLoesung(index,loesung, neuX, neuY, 1);			 

			// if (loesung noch nicht vollstndig) 
			if (!ausgangGefunden(neuX, neuY)) {
				// rekursiver Aufruf von FindeLoesung
				if (FindeLoesung(index+1, loesung, neuX, neuY)) {
					// Lsung gefunden	
					return true;
				} else {
					// Wir sind in einer Sackgasse:
					// Mache schritt rckgngig: Backtracking
					loesung.feld[neuX][neuY] = SACKGASSE;
					
					// Visualisierung
					visualisiereFindeLoesung(index,loesung, neuX, neuY, 2);
					visualisiereFindeLoesung(index-1,loesung, aktX, aktY, 3);						
				}
			} else return true; // Lsung gefunden -> fertig		
		}	
	}
	return false; // keine Lsung gefunden
}
/*** Ende des Codes vom Vortrag ***/

/**************************** Zusatzcode *******************************/
// Konstruktor
public Labyrinth() {
// Datenstrukturen initialisieren und GUI erstellen
	mazeGenerator = new MazeGenerator(MAXSIZE, MAXSIZE);	
	newLabyrinth(K,L); // Datenstrukturen initialisieren

	setLayout(new BorderLayout());
	add(labyrinthCanvas = new LabyrinthCanvas(this), BorderLayout.CENTER);
	
	// MenuBar mit Menu erzeugen
	MenuBar mb		= new MenuBar();
	AnimationMenu	= new Menu("Animation");
	LabyrinthMenu	= new Menu("Labyrinth");	
	DelayMenu		= new Menu("Animationsverzgerung"); 
	HelpMenu		= new Menu("Hilfe");

	// Checkbox Menus fuer Labyrinthgroesse hinzufuegen
	size = new CheckboxMenuItem[MAXSIZE];
	for (int i = MINSIZE; i < MAXSIZE; i++) {
		size[i-MINSIZE] = new CheckboxMenuItem(i + "x" + i,false);
		size[i-MINSIZE].addItemListener(this);
		LabyrinthMenu.add(size[i-MINSIZE]);
	}
	size[K-MINSIZE].setState(true);
	
	// Checkbox Menus fuer Animationsverzoegerung hinzufuegen
	delay = new CheckboxMenuItem[(MAXDELAY - MINDELAY) / DELTADELAY + 1];
	for (int i = MINDELAY; i <= MAXDELAY; i = i + DELTADELAY) {
		delay[(i - MINDELAY) / DELTADELAY] = new CheckboxMenuItem(i + " ms",false);
		delay[(i - MINDELAY) / DELTADELAY].addItemListener(this);
		DelayMenu.add(delay[(i - MINDELAY) / DELTADELAY]);
	}
	delay[(cur_delay - MINDELAY) / DELTADELAY].setState(true);

	// MenuItems zum Menu erzeugen
	newLabyrinth = new MenuItem("Neues Labyrinth");
	solve        = new MenuItem("Weg finden mit Animation");
	resetLabyrinth = new MenuItem("Weg aus Labyrinth entfernen");
	quit         = new MenuItem("Beenden");
	about		 = new MenuItem("About ...");
	
	// Ereignisbehandlung hinzufuegen zu MenuItems
	newLabyrinth.addActionListener(this);
	solve.addActionListener(this);
	resetLabyrinth.addActionListener(this);
	quit.addActionListener(this);
	about.addActionListener(this);
	
	// MenuItems zum Menu hinzufuegen
	AnimationMenu.add(newLabyrinth);
	AnimationMenu.add(solve);
	AnimationMenu.add(resetLabyrinth);
	AnimationMenu.addSeparator();
	AnimationMenu.add(quit);
	HelpMenu.add(about);
	
	// Menu zur Menuleiste hinzufuegen	
	mb.add(AnimationMenu);
	mb.add(LabyrinthMenu);
	mb.add(DelayMenu);
	mb.add(HelpMenu);
	setMenuBar(mb);
	
	// Auf Schliessen des Fentsers reagieren
	addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                if (inAnApplet) { 
					dispose(); 
				} else {
                	System.exit(0);
				}
            }
     });

}

public void visualisiereFindeLoesung(int index, Lsg loesung, int x, int y, int mode) {
// Visualisierung von FindeLoesung
	Graphics g = labyrinthCanvas.getGraphics();
	
	int w = labyrinthCanvas.getSize().width;
    int h = labyrinthCanvas.getSize().height;
	
	int xoff = 5, yoff = 5;
	int xsize = (w - 2*xoff) / K;
	int ysize = (h - 2*yoff) / (L+LabyrinthCanvas.legendSize);
	xoff = (w - K * xsize) / 2;
	yoff = (h - (L+LabyrinthCanvas.legendSize) * ysize) / 2;
	g.translate(xoff,yoff);
	
	// Loesche Rahmen des letzten aktuellen Feldes
	if((lastX != -1) && (lastY != -1)) {
		// Loesche Rahmen um aktuelles Feld nach einer Pause
		// if (lastX == zielX && lastX == zielY)
		// 	g.setColor(Color.green); /* Zielpunkt wiederherstellen */
		// else
			g.setColor(Color.white); 
		g.drawRect(lastX*xsize + 4, lastY*ysize + 4, xsize-9, ysize-9);
		g.drawRect(lastX*xsize + 3, lastY*ysize + 3, xsize-7, ysize-7);	
	};
	
	// Zeichne Rahmen um aktuelles Feld
	g.setColor(Color.black);
	g.drawRect(x*xsize + 4, y*ysize + 4, xsize-9, ysize-9);
	g.drawRect(x*xsize + 3, y*ysize + 3, xsize-7, ysize-7);	
	lastX = x;
	lastY = y;
	
	if (mode == 1) {
		if ((x != startX) || (y != startY)) {
			if ((x != zielX) || (y != zielY)) {
				if (loesung.feld[x][y] >= 0) {
					// Hellgraues Rechteck zeichnen		
					g.setColor(Color.lightGray); 
					g.fillRect(x*xsize + 5, y*ysize + 5, xsize-10, ysize-10);	

					// Beschrifte mit Schrittzahl
					mode = 3;
				}
			}
		}
	} 
	
	if (mode == 2) {
		// Markiere Sackgasse
		g.setColor(Color.orange);
		g.fillRect(x*xsize + 5, y*ysize + 5, xsize-10, ysize-10);	
	}
	
	if (mode == 3) {
		// Beschrifte mit Schrittzahl
		g.setColor(Color.black);
		g.setFont(new Font("Helvetica", Font.PLAIN, ysize/2));
		String str = "" + index;
		int strWidth = g.getFontMetrics().stringWidth(str);
		int xLoc = x*xsize+xsize/2-strWidth/2;
		g.drawString(str, xLoc, y*ysize+ysize/2+ysize/2/2);	
	}
	
	// Pause 
	sleep(cur_delay);
}


public void sleep(long milliseconds) {
// Wartet die angegebene Anzahl Millisekunden
	Date date;
	long now, start;
	
	date = new Date();
	start = date.getTime();
	now = start;
	while (start + milliseconds > now) {
		/* busy waiting */
		date = new Date();
		now = date.getTime();
	}
}

public void actionPerformed(ActionEvent e) {
// Menu-Ereignisse behandeln
	if (e.getSource() == about) {
		String text = "Backtracking im Labyrinth\n";
		text += "(c) Juli 2000, Thomas Duebendorfer, thomas@duebendorfer.ch\n";
		text += "Programm und Quellcode zu Lehrzwecken frei verwendbar.\n";
		text += "Kommerzielle Verwendung nur mit schriftlicher Einwilligung.\n\n";
		text += "Algorithmus:\n";
		text += "Es wird versucht, den Weg nach 1. oben, 2. rechts, 3. unten ";
		text += "und 4. links zu erweitern. Sackgassen werden markiert und ";
		text += "die Schritte mit Backtracking zurckgenommen.";
		
		SimpleDialog dlg = new SimpleDialog(this, "About", true, text, "OK");
		dlg.setVisible(true);
	}
	if (e.getSource() == newLabyrinth) {
		newLabyrinth(K,L);
		labyrinthCanvas.repaint();
	}
	if (e.getSource() == solve) {
		solve();
	}		
	if (e.getSource() == resetLabyrinth) {
		for(int x = 0; x<K; x++) {
			for(int y = 0; y<L; y++) {
				loesung.feld[x][y] = LEER;
			}
		}
		labyrinthCanvas.repaint();
	}
	if (e.getSource() == quit) { System.exit(0); }
}

public void itemStateChanged(ItemEvent e) {
	// Testen, ob Brettgroesse durch Menuauswahl veraendert wurde
	for (int i = MINSIZE; i < MAXSIZE; i++) {
		if (e.getSource() == size[i-MINSIZE]) {
			size[K-MINSIZE].setState(false);
			size[i-MINSIZE].setState(true);
			newLabyrinth(i,i);
			labyrinthCanvas.repaint(); 
			break; // end for loop
		}
	}

	// Testen, ob Animationsverzoegerung durch Menuauswahl veraendert wurde	
	for (int i = MINDELAY; i <= MAXDELAY; i = i + DELTADELAY) {
		if (e.getSource() == delay[(i - MINDELAY) / DELTADELAY] ) {
			delay[(cur_delay - MINDELAY) / DELTADELAY].setState(false);
			delay[(i - MINDELAY) / DELTADELAY].setState(true);
			cur_delay = i;
			break;
		}
	}
}

void solve() {
	// Animation des Algorithmus' starten
	setTitle("Backtracking im Labyrinth - Animation ...");
			
	// Disable Menus during animation
	AnimationMenu.setEnabled(false);
	LabyrinthMenu.setEnabled(false);
	DelayMenu.setEnabled(false);
	HelpMenu.setEnabled(false);
	
	labyrinthCanvas.repaint();
	// Startfeld markieren
	loesung.feld[startX][startY] = 1;
	visualisiereFindeLoesung(1,loesung,startX,startY,3);
	if (FindeLoesung(2, loesung, startX, startY)) {
		// Die gefundene Lsung steht in loesung als Schrittzahlen
		// auf den jeweiligen Feldern:
		System.out.println("Weg gefunden!");
		setTitle("Backtracking im Labyrinth - Weg gefunden");
		for(int y = 0; y < L; y++) {
			for(int x = 0; x < K; x++) {
				String s = new String("    " + loesung.feld[x][y]);
				System.out.print(s.substring(s.length()-4) + ",");
			}
			System.out.println("");
		}
	} else {
		// Keine Lsung gefunden!
		System.out.println("Kein Weg!");
		setTitle("Backtracking im Labyrinth - Keinen Weg gefunden");
	}
	repaint();
	
	// Reenable Menus during animation
	AnimationMenu.setEnabled(true);
	LabyrinthMenu.setEnabled(true);
	DelayMenu.setEnabled(true);
	HelpMenu.setEnabled(true);	
}
	
boolean hatWand(int fromX, int fromY, int neuX, int neuY, int schritt) {
// Liefert true, wenn zwischen den Punkten (fromX, fromY) und (neuX, neuY)
// eine Wand liegt, die mit 'schritt' durchschritten wuerde
// Wir haben nur obere (UP) und linke (LEFT) Waende
	boolean wand = false;
	switch( schritt) {
		case OBEN:	if ((labyrinth.feld[fromX][fromY] & UP) != 0) wand = true;
					break;
		case RECHTS:	if ((labyrinth.feld[neuX][neuY]   & LEFT) != 0) wand = true;
					break;
		case UNTEN:	if ((labyrinth.feld[neuX][neuY]   & UP) != 0) wand = true;
					break;								
		case LINKS:	if ((labyrinth.feld[fromX][fromY] & LEFT) != 0) wand = true;
					break;
	}
	return wand;
}

boolean ausgangGefunden(int x, int y) {
// Liefert true, wenn der Ausgang erreicht wurde
	return ((x == zielX) && (y == zielY));
}

void newLabyrinth(int k, int l) {
// Neues Labyrinth erzeugen und Variable "loesung" zuruecksetzen
	int x, y;
	K = k; 
	L = l;
	
	labyrinth = new Lsg(K+1,L+1); // Labyrinth erzeugen
	loesung   = new Lsg(K,L); // Datenstruktur fuer Loesung erzeugen

	for(int i=0; i<K; i++) {
		for(int j=0; j<L; j++) {
			loesung.feld[i][j] = 0;
			labyrinth.feld[i][j] = 0;
		}
	}

	// Zufaelliges, aber perfektes Labyrinth berechnen
	mazeGenerator.newMaze(K, L);
	startX = mazeGenerator.startX; startY = mazeGenerator.startY;
	zielX  = mazeGenerator.targetX; zielY = mazeGenerator.targetY;
	
	// Wandcodierung vereinfachen fuer schnellen Zugriff
	for (y = 0; y <= L; y++) {
		for (x = 0; x <= K; x++) {
			if (mazeGenerator.rgfCell[2*y*mazeGenerator.xmaz + (x*2 + 1)])
				labyrinth.feld[x][y] |= UP; 
		}
		for (x = 0; x <= K; x++) {
			if (mazeGenerator.rgfCell[(y*2 + 1)*mazeGenerator.xmaz + x*2])
				labyrinth.feld[x][y] |= LEFT;
		}
	}
	
	setTitle("Backtracking im Labyrinth");
	lastX = -1;
	lastY = -1;
}
	
public static void main(String[] args) {
// Hauptprogramm, falls Labyrinth als Java application gestartet wird
    Labyrinth window = new Labyrinth();
	window.inAnApplet = false;

    window.setTitle("Backtracking im Labyrinth");
    window.pack(); 
	window.setVisible(true);
}


}