// LabyrinthCanvas.java
//
// (c) Thomas Duebendorfer, Aug 9, 2000
// Labyrinth zeichnen vor und nach Animation

import java.awt.*;

//Subklasse von Canvas, welche etwas in den Canvas zeichnet
class LabyrinthCanvas extends Canvas {
	Labyrinth labyrinth;
	final static int legendSize = 3; /* in ysize units */
	
	public LabyrinthCanvas(Labyrinth labyrinth) {
		super();
		this.labyrinth = labyrinth;
	}
	
    public void paint(Graphics g) {
        int w = getSize().width;
        int h = getSize().height;
		
		/* zeichne Labyrinth */
		int xoff = 5, yoff = 5;
		int xsize = (w - 2*xoff) / labyrinth.K;
		int ysize = (h - 2*yoff) / (labyrinth.L+legendSize);
		xoff = (w - labyrinth.K * xsize) / 2;
		yoff = (h - (labyrinth.L+legendSize) * ysize) / 2;
		g.translate(xoff,yoff);
		
		g.setColor(Color.black);
		for (int y = 0; y <= labyrinth.L; y++) {
			for (int x = 0; x <= labyrinth.K; x++) {
				int aktFeld = labyrinth.labyrinth.feld[x][y];
				if ((aktFeld & labyrinth.UP)    != 0) { g.drawLine(x*xsize,y*ysize,(x+1)*xsize,y*ysize); }
				if ((aktFeld & labyrinth.LEFT)  != 0) { g.drawLine(x*xsize,y*ysize,x*xsize,(y+1)*ysize); }												 
				
				if ((x == labyrinth.K) || (y == labyrinth.L))
					continue;
					
				if (labyrinth.loesung.feld[x][y] == labyrinth.SACKGASSE) {
					g.setColor(Color.orange);
					g.fillRect(x*xsize + 5, y*ysize + 5, xsize-10, ysize-10);
				}	
					
				if (labyrinth.loesung.feld[x][y] > 0) {
					if ((x != labyrinth.startX) || (y != labyrinth.startY)) {
						g.setColor(Color.lightGray);
						g.fillRect(x*xsize + 5, y*ysize + 5, xsize-10, ysize-10);
					}
				}				

				if ((labyrinth.zielX != labyrinth.startX) || 
					(labyrinth.zielY != labyrinth.startY))   {
					
					/* Markiere Startpunkt */
					g.setColor(Color.blue);
					if (x == labyrinth.startX && y == labyrinth.startY) {
						g.fillRect(x*xsize + 5, y*ysize + 5, xsize-10, ysize-10);
					}
									
					/* Markiere Zielpunkt */
					g.setColor(Color.green);
					if (x == labyrinth.zielX && y == labyrinth.zielY) {
						g.fillRect(x*xsize + 5, y*ysize + 5, xsize-10, ysize-10);
					}
				}
	
				g.setColor(Color.black);
				
				if (labyrinth.loesung.feld[x][y] > 0) {
					// Beschrifte mit Schrittzahl
					g.setFont(new Font("Helvetica", Font.PLAIN, ysize/2));
					String str = "" + labyrinth.loesung.feld[x][y];
					int strWidth = g.getFontMetrics().stringWidth(str);
					int xLoc = x*xsize+xsize/2-strWidth/2;
					g.drawString(str, xLoc, y*ysize+ysize/2+ysize/2/2);	
				}
			}
		}
		drawLegend(g, 0, labyrinth.L, xsize, ysize, xoff);
    }

	public void drawLegend(Graphics g, int x, int y, int xsize, int ysize, int xoff) {
		g.setFont(new Font("Helvetica", Font.PLAIN, ysize/2));
		int deltaY = (int)(3.0/4.0*(double)(ysize));	
		
		/* Farbige Felder zeichnen */
		g.setColor(Color.blue);
		g.fillRect(x*xsize + 3, y*ysize + 5 + deltaY, xsize-10, ysize-10);
		g.setColor(Color.green);
		g.fillRect(x*xsize + 3, (y+1)*ysize + 5  + deltaY, xsize-10, ysize-10);
	
		g.setColor(Color.lightGray);
		g.fillRect((x+labyrinth.K/2)*xsize + 5, y*ysize + 6  + deltaY, xsize-10, ysize-10);
		g.setColor(Color.orange);
		g.fillRect((x+labyrinth.K/2)*xsize + 5, (y+1)*ysize + 6  + deltaY, xsize-10, ysize-10);
		
		/* Beschriftung */
		g.setColor(Color.black);
		g.drawString("Start", (x+1)*xsize+xsize/2, y*ysize+2*deltaY);		
		g.drawString("Ziel", (x+1)*xsize+xsize/2, (y+1)*ysize+2*deltaY);				
		g.drawString("besucht", (x+labyrinth.K/2+1)*xsize+xsize/2, y*ysize+2*deltaY);		
		g.drawString("Sackgasse", (x+labyrinth.K/2+1)*xsize+xsize/2, (y+1)*ysize+2*deltaY);
	}
	
    //Ntig, damit Canvas sicher gezeichnet wird
    public Dimension getMinimumSize() {
        return new Dimension(360,460);
    }

    //Ntig, damit Canvas sicher gezeichnet wird
    public Dimension getPreferredSize() {
        return getMinimumSize();
    }
}
